package com.abssqr.plat.biz.api.ast;

import java.io.UnsupportedEncodingException;
import java.util.List;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;

import com.abssqr.plat.biz.support.ServiceSupport;
import com.abssqr.plat.common.dal.mysql.auto.paging.AstPackAddLoanPage;
import com.abssqr.plat.common.dal.mysql.auto.paging.AstPackLoanPage;
import com.abssqr.plat.common.dal.mysql.auto.paging.AstPoolLoanPage;
import com.abssqr.plat.common.dal.mysql.auto.paging.PackAstPage;
import com.abssqr.plat.common.facade.param.ast.AstDownLoadParam;
import com.abssqr.plat.common.facade.param.ast.AstPageQryParam;
import com.abssqr.plat.common.facade.result.BasePage;
import com.abssqr.plat.common.facade.result.ast.AstLoanVO;
import com.abssqr.plat.common.facade.service.ast.AstService;
import com.abssqr.plat.common.facade.validation.Valid;
import com.abssqr.plat.common.model.convertor.web.AstLoanWebConvert;
import com.abssqr.plat.common.model.domain.ast.AstPack;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.ast.LoanRepository;
import com.abssqr.plat.core.service.ast.AstPackManager;
import com.abssqr.plat.core.service.ast.DownLoadAstManager;

import com.general.system.common.util.VarChecker;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
public class AstServiceImpl extends ServiceSupport implements AstService {
    protected final Logger LOGGER = LoggerFactory.getLogger(getClass());

    private final int limit=2000;

    @Autowired
    LoanRepository     loanRepository;
    @Autowired
    AstPackManager     astPackManager;
    @Autowired
    DownLoadAstManager downLoadAstManager;

    @Override
    public void downLoadAstFile(@Valid AstDownLoadParam astDownLoadParam, HttpServletResponse response) {

        PackAstPage packAstPage=new PackAstPage();
        packAstPage.setPackNo(astDownLoadParam.getPackCode());
        packAstPage.setCurrPageNo(1);
        packAstPage.setLimit(limit);
        List<String> loanFieldCodes = astDownLoadParam.getLoanFieldCodes();
        try {
            downLoadAstManager.writeData2Stream(packAstPage,loanFieldCodes,getOutputStream(astDownLoadParam.getPackCode(),response));
        } catch (Exception e) {
            LOGGER.error("写入数据失败错误信息[0]",e.getMessage());
            throw new AbssqrBizException("下载资产列表任务异常");
        }

    }

    /**
     * 查询资产列表
     *
     * @param astPageQryParam
     * @return
     */
    @Override
    public BasePage<AstLoanVO> poolAstList(AstPageQryParam astPageQryParam) {
        return super.doQry(astPageQryParam, () -> {
            AstPoolLoanPage astPoolLoanPage = new AstPoolLoanPage();
            astPoolLoanPage.setAstPoolNo(astPageQryParam.getAstPoolNo());
            astPoolLoanPage.setCurrPageNo(astPageQryParam.getPageNum());
            astPoolLoanPage.setLimit(astPageQryParam.getPageNum());
            astPoolLoanPage = loanRepository.getLoanPageByAstPoolNo(astPoolLoanPage);
            return AstLoanWebConvert.convert2AstPoolVOs(astPoolLoanPage);
        });
    }

    @Override
    public BasePage<AstLoanVO> packAstList(AstPageQryParam astPageQryParam) {
        VarChecker.checkNotEmpty(astPageQryParam.getPackNo(), "资产包编号不能为空");
        return super.doQry(astPageQryParam, () -> {
            AstPack astPack = astPackManager.get(astPageQryParam.getPackNo());
            AstPackLoanPage astPackLoanPage = new AstPackLoanPage();
            astPackLoanPage.setPackNo(astPack.getPackNo());
            astPackLoanPage = loanRepository.getLoanListForPack(astPackLoanPage);
            return AstLoanWebConvert.convert2AstPoolVOs(astPackLoanPage);
        });
    }

    @Override
    public BasePage<AstLoanVO> packAddAstList(AstPageQryParam astPageQryParam) {
        VarChecker.checkNotEmpty(astPageQryParam.getPackNo(), "资产包编号不能为空");
        return super.doQry(astPageQryParam, () -> {
            AstPack astPack = astPackManager.get(astPageQryParam.getPackNo());
            AstPackAddLoanPage astPackLoanPage = new AstPackAddLoanPage();
            astPackLoanPage.setAstPoolNo(astPack.getAstPoolNo());
            astPackLoanPage.setAstType(astPack.getAstType());
            astPackLoanPage.setScreenRuleList(astPack.getScreenRuleList());
            astPackLoanPage = loanRepository.getLoanPageForPackAddAst(astPackLoanPage);
            return AstLoanWebConvert.convert2AstPoolVOs(astPackLoanPage);
        });
    }

    private ServletOutputStream getOutputStream(String packNo,HttpServletResponse response) throws Exception{
        response.reset();
        response.setCharacterEncoding("UTF-8");
        response.setContentType("multipart/form-data");
        String outfileName = null;
        try {
            outfileName = new String((packNo+".xls").getBytes(), "ISO-8859-1");
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException(e.getMessage());
        }
        response.setHeader("Content-Disposition", "attachment;fileName=" + outfileName);

        return response.getOutputStream();

    }


}
